"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const enzyme_1 = require("enzyme");
const Chip_1 = require("../Chip");
const theme = {
    Chip: 'chip',
    ChipClickable: 'chipClickable',
    Image: 'chipImage',
    chipRemovable: 'chipRemovable',
    transparent: 'chipTransparent',
    Remove: 'chipRemove',
};
describe('<Chip />', () => {
    describe('image property', () => {
        describe('when not set', () => {
            it('should not have image element', () => {
                const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme }));
                expect(subject.find('img')).toHaveLength(0);
            });
        });
        describe('when set to valid image object', () => {
            const imageUrl = 'http://placehold.it/50x50';
            const altText = 'Hello World';
            const image = {
                url: imageUrl,
                alt: altText,
            };
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { image: image, theme: theme }));
            it('should have rendered 2 span element', () => {
                expect(subject.find('span')).toHaveLength(2);
            });
            it('should have default chip css class on span', () => {
                expect(subject.find('button').at(0).hasClass('chip')).toBe(true);
            });
            it('should have image element', () => {
                expect(subject.find('img')).toHaveLength(1);
            });
            it('should have image element with given image url', () => {
                expect(subject.find('img').prop('src')).toBe(imageUrl);
            });
            it('should have image element with given alternate text', () => {
                expect(subject.find('img').prop('alt')).toBe(altText);
            });
        });
        describe('when set to undefined', () => {
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { image: undefined, theme: theme }));
            it('should have rendered 2 span element', () => {
                expect(subject.find('span')).toHaveLength(2);
            });
            it('should have default chip css class on span', () => {
                expect(subject.find('button').at(0).hasClass('chip')).toBe(true);
            });
            it('should not have image element', () => {
                expect(subject.find('img')).toHaveLength(0);
            });
        });
    });
    describe('clickable property', () => {
        describe('when not set', () => {
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme }));
            it('should have rendered 2 span element', () => {
                expect(subject.find('span')).toHaveLength(2);
            });
            it('should have default chip css class on span', () => {
                expect(subject.find('button').at(0).hasClass('chip')).toBe(true);
            });
            it('should not have <a> element', () => {
                expect(subject.find('a')).toHaveLength(0);
            });
            it('should not have clickable chip css class on span', () => {
                expect(subject.find('span').at(0).hasClass('chipClickable')).toBe(false);
            });
        });
        describe('when set to true', () => {
            it('should have rendered 2 span elements', () => {
                const subject = enzyme_1.mount(React.createElement(Chip_1.default, { clickable: true, theme: theme }));
                expect(subject.find('span')).toHaveLength(2);
            });
            it('should have default chip css class on span', () => {
                const subject = enzyme_1.mount(React.createElement(Chip_1.default, { clickable: true, theme: theme }));
                expect(subject.find('button').at(0).hasClass('chip')).toBe(true);
            });
            it('should have <a> element', () => {
                const subject = enzyme_1.mount(React.createElement(Chip_1.default, { clickable: true, theme: theme }));
                expect(subject.find('a')).toHaveLength(1);
            });
            it('simulate chip click event', (event) => {
                const onChipClick = jest.fn(event);
                const subject = enzyme_1.mount(React.createElement(Chip_1.default, { clickable: true, onClick: onChipClick, theme: theme }));
                subject.find('a').simulate('click');
                expect(onChipClick).toHaveBeenCalledTimes(1);
            });
        });
        describe('when set to false', () => {
            it('should have rendered 2 span element', () => {
                const subject = enzyme_1.mount(React.createElement(Chip_1.default, { clickable: false, theme: theme }));
                expect(subject.find('span')).toHaveLength(2);
            });
            it('should have default chip css class on span', () => {
                const subject = enzyme_1.mount(React.createElement(Chip_1.default, { clickable: false, theme: theme }));
                expect(subject.find('button').at(0).hasClass('chip')).toBe(true);
            });
            it('should not have <a> element', () => {
                const subject = enzyme_1.mount(React.createElement(Chip_1.default, { clickable: false, theme: theme }));
                expect(subject.find('a')).toHaveLength(0);
            });
            it('should not have clickable chip css class on span', () => {
                const subject = enzyme_1.mount(React.createElement(Chip_1.default, { clickable: false, theme: theme }));
                expect(subject.find('span').at(0).hasClass('chipClickable')).toBe(false);
            });
        });
    });
    describe('removable property', () => {
        describe('when not set', () => {
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme }));
            it('should have rendered 2 span element', () => {
                expect(subject.find('span')).toHaveLength(2);
            });
            it('should have default chip css class on span', () => {
                expect(subject.find('button').at(0).hasClass('chip')).toBe(true);
            });
            it('should have <button> element', () => {
                expect(subject.find('button')).toHaveLength(1);
            });
            it('should not have removable chip css class on span', () => {
                expect(subject.find('span').at(0).hasClass('chipRemovable')).toBe(false);
            });
        });
        describe('when set to true', () => {
            it('should have rendered 3 span elements', () => {
                const onRemoveClick = jest.fn();
                const subject = enzyme_1.mount(React.createElement(Chip_1.default, { removable: true, theme: theme, onRemove: onRemoveClick }));
                expect(subject.find('span')).toHaveLength(3);
            });
            it('should have default chip css class on span', () => {
                const onRemoveClick = jest.fn();
                const subject = enzyme_1.mount(React.createElement(Chip_1.default, { removable: true, theme: theme, onRemove: onRemoveClick }));
                expect(subject.find('button').hasClass('chip')).toBe(true);
            });
            it('should have <button> element', () => {
                const onRemoveClick = jest.fn();
                const subject = enzyme_1.mount(React.createElement(Chip_1.default, { removable: true, theme: theme, onRemove: onRemoveClick }));
                expect(subject.find('button')).toHaveLength(1);
            });
            it('should have button remove css style', () => {
                const onRemoveClick = jest.fn();
                const subject = enzyme_1.mount(React.createElement(Chip_1.default, { removable: true, theme: theme, onRemove: onRemoveClick }));
                expect(subject.find('a').hasClass('chipRemove')).toBe(true);
            });
            it('simulate remove button click event', (event) => {
                const onRemoveClick = jest.fn(event);
                const subject = enzyme_1.mount(React.createElement(Chip_1.default, { removable: true, onRemove: onRemoveClick, theme: theme }));
                subject.find('a').simulate('click');
                expect(onRemoveClick).toHaveBeenCalledTimes(1);
            });
        });
        describe('when set to false', () => {
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { removable: false, theme: theme }));
            it('should have rendered 2 span elements', () => {
                expect(subject.find('span')).toHaveLength(2);
            });
            it('should have default chip css class on span', () => {
                expect(subject.find('button').at(0).hasClass('chip')).toBe(true);
            });
            it('should have <button> element', () => {
                expect(subject.find('button')).toHaveLength(1);
            });
            it('should not have removable chip css class on span', () => {
                expect(subject.find('span').at(0).hasClass('chipRemovable')).toBe(false);
            });
        });
    });
    describe('transparent property', () => {
        describe('when not set', () => {
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme }));
            it('should have rendered 2 span elements', () => {
                expect(subject.find('span')).toHaveLength(2);
            });
            it('should have default chip css class on span', () => {
                expect(subject.find('button').at(0).hasClass('chip')).toBe(true);
            });
            it('should not have transparent chip css class on span', () => {
                expect(subject.find('span').at(0).hasClass('chipTransparent')).toBe(false);
            });
        });
        describe('when set to true', () => {
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { transparent: true, theme: theme }));
            it('should have rendered 2 span elements', () => {
                expect(subject.find('span')).toHaveLength(2);
            });
            it('should have default chip css class on span', () => {
                expect(subject.find('button').at(0).hasClass('chip')).toBe(true);
            });
            it('should have transparent chip css class on span', () => {
                expect(subject.find('button').at(0).hasClass('chipTransparent')).toBe(true);
            });
        });
        describe('when set to false', () => {
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { transparent: false, theme: theme }));
            it('should have rendered 2 span elements', () => {
                expect(subject.find('span')).toHaveLength(2);
            });
            it('should have default chip css class on span', () => {
                expect(subject.find('button').at(0).hasClass('chip')).toBe(true);
            });
            it('should not have transparent chip css class on span', () => {
                expect(subject.find('span').at(0).hasClass('chipTransparent')).toBe(false);
            });
        });
    });
    describe('when clickable, removable, transparent chip with image is provided', () => {
        const image = {
            url: 'http://placehold.it/50x50',
            alt: 'Alternate Text',
        };
        it('should have rendered 3 span elements', () => {
            const onChipClick = jest.fn();
            const onRemoveClick = jest.fn();
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme, transparent: true, clickable: true, removable: true, image: image, onClick: onChipClick, onRemove: onRemoveClick }));
            expect(subject.find('span')).toHaveLength(3);
        });
        it('should have default chip css class on span', () => {
            const onChipClick = jest.fn();
            const onRemoveClick = jest.fn();
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme, transparent: true, clickable: true, removable: true, image: image, onClick: onChipClick, onRemove: onRemoveClick }));
            expect(subject.find('button').hasClass('chip')).toBe(true);
        });
        it('should have <a> element', () => {
            const onChipClick = jest.fn();
            const onRemoveClick = jest.fn();
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme, transparent: true, clickable: true, removable: true, image: image, onClick: onChipClick, onRemove: onRemoveClick }));
            expect(subject.find('a')).toHaveLength(2);
        });
        it('should have image element', () => {
            const onChipClick = jest.fn();
            const onRemoveClick = jest.fn();
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme, transparent: true, clickable: true, removable: true, image: image, onClick: onChipClick, onRemove: onRemoveClick }));
            expect(subject.find('img')).toHaveLength(1);
        });
        it('should have image element with given image url', () => {
            const onChipClick = jest.fn();
            const onRemoveClick = jest.fn();
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme, transparent: true, clickable: true, removable: true, image: image, onClick: onChipClick, onRemove: onRemoveClick }));
            expect(subject.find('img').prop('src')).toBe(image.url);
        });
        it('should have image element with given alternate text', () => {
            const onChipClick = jest.fn();
            const onRemoveClick = jest.fn();
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme, transparent: true, clickable: true, removable: true, image: image, onClick: onChipClick, onRemove: onRemoveClick }));
            expect(subject.find('img').prop('alt')).toBe(image.alt);
        });
        it('should have <button> element', () => {
            const onChipClick = jest.fn();
            const onRemoveClick = jest.fn();
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme, transparent: true, clickable: true, removable: true, image: image, onClick: onChipClick, onRemove: onRemoveClick }));
            expect(subject.find('button')).toHaveLength(1);
        });
        it('should have button remove css style', () => {
            const onChipClick = jest.fn();
            const onRemoveClick = jest.fn();
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme, transparent: true, clickable: true, removable: true, image: image, onClick: onChipClick, onRemove: onRemoveClick }));
            expect(subject.find('a').at(1).hasClass('chipRemove')).toBe(true);
        });
        it('should have transparent chip css class on span', () => {
            const onChipClick = jest.fn();
            const onRemoveClick = jest.fn();
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme, transparent: true, clickable: true, removable: true, image: image, onClick: onChipClick, onRemove: onRemoveClick }));
            expect(subject.find('button').at(0).hasClass('chipTransparent')).toBe(true);
        });
        it('simulate chip click event', (event) => {
            const onChipClick = jest.fn(event);
            const onRemoveClick = jest.fn(event);
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme, transparent: true, clickable: true, removable: true, image: image, onClick: onChipClick, onRemove: onRemoveClick }));
            subject.find('a').at(1).simulate('click');
            expect(onChipClick).toHaveBeenCalledTimes(1);
        });
        it('simulate remove button click event', (event) => {
            const onChipClick = jest.fn(event);
            const onRemoveClick = jest.fn(event);
            const subject = enzyme_1.mount(React.createElement(Chip_1.default, { theme: theme, transparent: true, clickable: true, removable: true, image: image, onClick: onChipClick, onRemove: onRemoveClick }));
            subject.find('a').at(1).simulate('click');
            expect(onRemoveClick).toHaveBeenCalledTimes(1);
        });
    });
});
