"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const enzyme_1 = require("enzyme");
const ChoiceList_1 = require("../ChoiceList");
const RadioButton_1 = require("../../RadioButton");
const Checkbox_1 = require("../../Checkbox");
const theme = {
    choiceList: 'choiceList',
    titleHidden: 'titleHidden',
    choices: 'choices',
    title: 'title',
};
describe('<ChoiceList />', () => {
    let choices;
    beforeEach(() => {
        choices = [
            { label: 'One', value: 'one' },
            { label: 'Two', value: 'two' },
            { label: 'Three', value: 'three' },
        ];
    });
    describe('when default props are provided', () => {
        it('should verify renders a fieldset', () => {
            const element = enzyme_1.mount(React.createElement(ChoiceList_1.default, { selected: [], choices: choices, theme: theme }));
            expect(element.find('fieldset').exists()).toBe(true);
            expect(element.find('fieldset').hasClass('choiceList')).toBe(true);
        });
    });
    describe('title property', () => {
        describe('when not set', () => {
            it('should verify not renders a legend for the fieldset', () => {
                const element = enzyme_1.mount(React.createElement(ChoiceList_1.default, { selected: [], choices: choices, theme: theme }));
                expect(element.prop('legend')).toBeUndefined();
            });
        });
        describe('when set', () => {
            it('should verify renders a legend for the fieldset', () => {
                const element = enzyme_1.mount(React.createElement(ChoiceList_1.default, { title: "My title", selected: [], choices: choices, theme: theme }));
                expect(element.find('legend')).toHaveLength(1);
                expect(element.find('legend').at(0).hasClass('title')).toBe(true);
                expect(element.find('legend').text()).toBe('My title');
            });
        });
    });
    describe('choices property', () => {
        it('shoild verify renders a choice with the label and value properties', () => {
            const choiceElements = enzyme_1.mount(React.createElement(ChoiceList_1.default, { selected: [], choices: choices, theme: theme })).find(RadioButton_1.default);
            choiceElements.forEach((choiceElement, index) => {
                expect(choiceElement.prop('label')).toBe(choices[index].label);
                expect(choiceElement.prop('value')).toBe(choices[index].value);
            });
        });
    });
    describe('selected property', () => {
        it('sets the appropriate choices to be selected', () => {
            const selectedIndexes = [0, 2];
            const selected = selectedIndexes.map(index => choices[index].value);
            const choiceElements = enzyme_1.mount(React.createElement(ChoiceList_1.default, { selected: selected, choices: choices, theme: theme })).find(RadioButton_1.default);
            choiceElements.forEach((choiceElement, index) => {
                expect(choiceElement.prop('checked')).toBe(selectedIndexes.includes(index));
            });
        });
    });
    describe('onChange()', () => {
        it('is called with the newly-selected choices', () => {
            let selected = [choices[0].value];
            const spy = jest.fn((newSelected) => {
                selected = newSelected;
            });
            const choiceList = enzyme_1.mount(React.createElement(ChoiceList_1.default, { allowMultiple: true, onChange: spy, selected: selected, choices: choices, theme: theme }));
            const choiceElements = choiceList.find(Checkbox_1.default);
            changeCheckedForChoice(choiceElements.at(1), true);
            expect(spy).toHaveBeenLastCalledWith(['one', 'two']);
            choiceList.setProps({ selected });
            changeCheckedForChoice(choiceElements.at(2), true);
            expect(spy).toHaveBeenLastCalledWith(['one', 'two', 'three']);
            choiceList.setProps({ selected });
            changeCheckedForChoice(choiceElements.at(0), false);
            expect(spy).toHaveBeenLastCalledWith(['two', 'three']);
            choiceList.setProps({ selected });
        });
        function changeCheckedForChoice(choice, checked, triggerChange = true) {
            const input = choice.find('input');
            input.node.checked = checked;
            if (triggerChange) {
                input.simulate('change');
            }
        }
    });
    describe('name property', () => {
        it('provides a unique name when none is provided', () => {
            const choiceElements = enzyme_1.shallow(React.createElement(ChoiceList_1.default, { selected: [], choices: choices, theme: theme })).find(RadioButton_1.default);
            let name;
            choiceElements.forEach((choiceElement) => {
                const choiceName = choiceElement.prop('name');
                if (name == null) {
                    name = choiceName;
                }
                else {
                    expect(choiceName).toBe(name);
                }
                expect(typeof choiceName).toBe('string');
            });
        });
        it('uses the same name for every choice', () => {
            const name = 'MyChoiceList';
            const choiceElements = enzyme_1.shallow(React.createElement(ChoiceList_1.default, { name: name, selected: [], choices: choices, theme: theme })).find(RadioButton_1.default);
            choiceElements.forEach((choiceElement) => {
                expect(choiceElement.prop('name')).toBe(name);
            });
        });
        it('postpends [] when multiple options are allowed', () => {
            const name = 'MyChoiceList';
            const choiceElements = enzyme_1.shallow(React.createElement(ChoiceList_1.default, { allowMultiple: true, name: name, selected: [], choices: choices, theme: theme })).find(RadioButton_1.default);
            choiceElements.forEach((choiceElement) => {
                expect(choiceElement.prop('name')).toBe(`${name}[]`);
            });
        });
    });
    describe('allowMultiple property', () => {
        it('renders a radio button for each option when allowMultiple is not true', () => {
            let element = enzyme_1.mount(React.createElement(ChoiceList_1.default, { selected: [], choices: choices, theme: theme }));
            expect(element.find(RadioButton_1.default).length).toBe(choices.length);
            expect(element.find(Checkbox_1.default).exists()).toBe(false);
            element = enzyme_1.mount(React.createElement(ChoiceList_1.default, { selected: [], choices: choices, allowMultiple: false, theme: theme }));
            expect(element.find(RadioButton_1.default).length).toBe(choices.length);
            expect(element.find(Checkbox_1.default).exists()).toBe(false);
        });
        it('renders a checkbox each option when allowMultiple is true', () => {
            const element = enzyme_1.mount(React.createElement(ChoiceList_1.default, { allowMultiple: true, selected: [], choices: choices, theme: theme }));
            expect(element.find(RadioButton_1.default).exists()).toBe(false);
            expect(element.find(Checkbox_1.default).length).toBe(choices.length);
        });
    });
    describe('titlehidden property', () => {
        describe('when not set', () => {
            it('should verify not renders a titlehidden for the choicelist', () => {
                const element = enzyme_1.mount(React.createElement(ChoiceList_1.default, { selected: [], choices: choices, theme: theme }));
                expect(element.prop('titleHidden')).toBeUndefined();
                expect(element.find('fieldset').hasClass('titleHidden')).toBe(false);
            });
        });
        describe('when  set as true', () => {
            it('should verify renders a titlehidden for the choicelist when set as true', () => {
                const element = enzyme_1.mount(React.createElement(ChoiceList_1.default, { title: "My title", titleHidden: true, selected: [], choices: choices, theme: theme }));
                expect(element.prop('titleHidden')).toBe(true);
                expect(element.find('fieldset').hasClass('titleHidden')).toBe(true);
            });
        });
        describe('when  set as false', () => {
            it('should verify renders a titlehidden for the choicelist when set as false', () => {
                const element = enzyme_1.mount(React.createElement(ChoiceList_1.default, { title: "My title", titleHidden: false, selected: [], choices: choices, theme: theme }));
                expect(element.prop('titleHidden')).toBe(false);
                expect(element.find('fieldset').hasClass('titleHidden')).toBe(false);
            });
        });
    });
});
