"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const prop_types_1 = require("prop-types");
const airbnb_prop_types_1 = require("airbnb-prop-types");
const classnames_1 = require("classnames");
const throttle_1 = require("lodash/throttle");
const is_touch_device_1 = require("is-touch-device");
const react_css_themr_1 = require("react-css-themr");
const ThemeIdentifiers_1 = require("./../../ThemeIdentifiers");
const baseTheme = require("./../style/style.scss");
const propTypes = airbnb_prop_types_1.forbidExtraProps({
    id: prop_types_1.default.string.isRequired,
    placeholder: prop_types_1.default.string,
    displayValue: prop_types_1.default.string,
    inputValue: prop_types_1.default.string,
    screenReaderMessage: prop_types_1.default.string,
    focused: prop_types_1.default.bool,
    disabled: prop_types_1.default.bool,
    required: prop_types_1.default.bool,
    readOnly: prop_types_1.default.bool,
    showCaret: prop_types_1.default.bool,
    onChange: prop_types_1.default.func,
    onFocus: prop_types_1.default.func,
    onKeyDownShiftTab: prop_types_1.default.func,
    onKeyDownTab: prop_types_1.default.func,
    onKeyDownArrowDown: prop_types_1.default.func,
    onKeyDownQuestionMark: prop_types_1.default.func,
    // accessibility
    isFocused: prop_types_1.default.bool,
    theme: prop_types_1.default.any,
});
const defaultProps = {
    placeholder: 'Select Date',
    displayValue: '',
    inputValue: '',
    screenReaderMessage: '',
    focused: false,
    disabled: false,
    required: false,
    readOnly: null,
    showCaret: false,
    onChange() { },
    onFocus() { },
    onKeyDownShiftTab() { },
    onKeyDownTab() { },
    onKeyDownArrowDown() { },
    onKeyDownQuestionMark() { },
    // accessibility
    isFocused: false,
};
class DateInput extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.state = {
            dateString: '',
            isTouchDevice: false,
        };
        this.onChange = this.onChange.bind(this);
        this.onKeyDown = this.onKeyDown.bind(this);
    }
    componentDidMount() {
        this.setState({ isTouchDevice: is_touch_device_1.default() });
    }
    componentWillReceiveProps(nextProps) {
        if (!this.props.displayValue && nextProps.displayValue) {
            this.setState({
                dateString: '',
            });
        }
    }
    componentDidUpdate(prevProps) {
        const { focused, isFocused } = this.props;
        if (prevProps.focused === focused && prevProps.isFocused === isFocused)
            return;
        if (focused && isFocused) {
            this.inputRef.focus();
            this.inputRef.select();
        }
        else {
            this.inputRef.blur();
        }
    }
    onChange(e) {
        const { onChange, onKeyDownQuestionMark } = this.props;
        const dateString = e.target.value;
        // In Safari, onKeyDown does not consistently fire ahead of onChange. As a result, we need to
        // special case the `?` key so that it always triggers the appropriate callback, instead of
        // modifying the input value
        if (dateString[dateString.length - 1] === '?') {
            onKeyDownQuestionMark(e);
        }
        else {
            this.setState({ dateString });
            onChange(dateString);
        }
    }
    onKeyDown(e) {
        e.stopPropagation();
        const { onKeyDownShiftTab, onKeyDownTab, onKeyDownArrowDown, onKeyDownQuestionMark, } = this.props;
        const { key } = e;
        if (key === 'Tab') {
            if (e.shiftKey) {
                onKeyDownShiftTab(e);
            }
            else {
                onKeyDownTab(e);
            }
        }
        else if (key === 'ArrowDown') {
            onKeyDownArrowDown(e);
        }
        else if (key === '?') {
            e.preventDefault();
            onKeyDownQuestionMark(e);
        }
    }
    render() {
        const { dateString, isTouchDevice: isTouch, } = this.state;
        const { id, placeholder, displayValue, inputValue, screenReaderMessage, focused, showCaret, onFocus, disabled, required, readOnly, theme, } = this.props;
        const displayText = displayValue || inputValue || dateString || placeholder || '';
        const value = inputValue || displayValue || dateString || '';
        const screenReaderMessageId = `DateInput__screen-reader-message-${id}`;
        return (react_1.default.createElement("div", { className: classnames_1.default(theme['DateInput'], {
                [theme['DateInput--with-caret']]: showCaret && focused,
                [theme['DateInput--disabled']]: disabled,
            }) },
            react_1.default.createElement("input", { "aria-label": placeholder, className: theme["DateInput__input needsclick"], type: "text", id: id, name: id, ref: (ref) => { this.inputRef = ref; }, value: value, onChange: this.onChange, onKeyDown: throttle_1.default(this.onKeyDown, 300), onFocus: onFocus, placeholder: placeholder, autoComplete: "off", disabled: disabled, readOnly: typeof readOnly === 'boolean' ? readOnly : isTouch, required: required, "aria-describedby": screenReaderMessage && screenReaderMessageId }),
            screenReaderMessage && (react_1.default.createElement("p", { id: screenReaderMessageId, className: theme["screen-reader-only"] }, screenReaderMessage)),
            react_1.default.createElement("div", { className: classnames_1.default(theme['DateInput__display-text'], {
                    [theme['DateInput__display-text--has-input']]: !!value,
                    [theme['DateInput__display-text--focused']]: focused,
                    [theme['DateInput__display-text--disabled']]: disabled,
                }) }, displayText)));
    }
}
exports.default = react_css_themr_1.themr(ThemeIdentifiers_1.DATEPICKER, baseTheme)(DateInput);
DateInput.propTypes = propTypes;
DateInput.defaultProps = defaultProps;
