"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const prop_types_1 = require("prop-types");
const react_addons_shallow_compare_1 = require("react-addons-shallow-compare");
const react_dom_1 = require("react-dom");
const airbnb_prop_types_1 = require("airbnb-prop-types");
const moment_1 = require("moment");
const classnames_1 = require("classnames");
const throttle_1 = require("lodash/throttle");
const is_touch_device_1 = require("is-touch-device");
const react_css_themr_1 = require("react-css-themr");
const defaultPhrases_1 = require("../defaultPhrases");
const getPhrasePropTypes_1 = require("../utils/getPhrasePropTypes");
const OutsideClickHandler_1 = require("./OutsideClickHandler");
const CalendarMonthGrid_1 = require("./CalendarMonthGrid");
const DayPickerNavigation_1 = require("./DayPickerNavigation");
const DayPickerKeyboardShortcuts_1 = require("./DayPickerKeyboardShortcuts");
const getTransformStyles_1 = require("../utils/getTransformStyles");
const getCalendarMonthWidth_1 = require("../utils/getCalendarMonthWidth");
const getActiveElement_1 = require("../utils/getActiveElement");
const isDayVisible_1 = require("../utils/isDayVisible");
const ScrollableOrientationShape_1 = require("../shapes/ScrollableOrientationShape");
const DayOfWeekShape_1 = require("../shapes/DayOfWeekShape");
const Constants_1 = require("../Constants");
const ThemeIdentifiers_1 = require("./../../ThemeIdentifiers");
const baseTheme = require("./../style/style.scss");
const MONTH_PADDING = 23;
const DAY_PICKER_PADDING = 9;
const PREV_TRANSITION = 'prev';
const NEXT_TRANSITION = 'next';
const propTypes = airbnb_prop_types_1.forbidExtraProps({
    // calendar presentation props
    enableOutsideDays: prop_types_1.default.bool,
    numberOfMonths: prop_types_1.default.number,
    orientation: ScrollableOrientationShape_1.default,
    withPortal: prop_types_1.default.bool,
    onOutsideClick: prop_types_1.default.func,
    hidden: prop_types_1.default.bool,
    initialVisibleMonth: prop_types_1.default.func,
    firstDayOfWeek: DayOfWeekShape_1.default,
    renderCalendarInfo: prop_types_1.default.func,
    hideKeyboardShortcutsPanel: prop_types_1.default.bool,
    daySize: airbnb_prop_types_1.nonNegativeInteger,
    isRTL: prop_types_1.default.bool,
    // navigation props
    navPrev: prop_types_1.default.node,
    navNext: prop_types_1.default.node,
    onPrevMonthClick: prop_types_1.default.func,
    onNextMonthClick: prop_types_1.default.func,
    onMultiplyScrollableMonths: prop_types_1.default.func,
    // month props
    renderMonth: prop_types_1.default.func,
    // day props
    modifiers: prop_types_1.default.object,
    renderDay: prop_types_1.default.func,
    onDayClick: prop_types_1.default.func,
    onDayMouseEnter: prop_types_1.default.func,
    onDayMouseLeave: prop_types_1.default.func,
    // accessibility props
    isFocused: prop_types_1.default.bool,
    getFirstFocusableDay: prop_types_1.default.func,
    onBlur: prop_types_1.default.func,
    showKeyboardShortcuts: prop_types_1.default.bool,
    // internationalization
    monthFormat: prop_types_1.default.string,
    phrases: prop_types_1.default.shape(getPhrasePropTypes_1.default(defaultPhrases_1.DayPickerPhrases)),
    theme: prop_types_1.default.any,
});
exports.defaultProps = {
    // calendar presentation props
    enableOutsideDays: false,
    numberOfMonths: 2,
    orientation: Constants_1.HORIZONTAL_ORIENTATION,
    withPortal: false,
    onOutsideClick() { },
    hidden: false,
    initialVisibleMonth: () => moment_1.default(),
    firstDayOfWeek: null,
    renderCalendarInfo: null,
    hideKeyboardShortcutsPanel: false,
    daySize: Constants_1.DAY_SIZE,
    isRTL: false,
    // navigation props
    navPrev: null,
    navNext: null,
    onPrevMonthClick() { },
    onNextMonthClick() { },
    onMultiplyScrollableMonths() { },
    // month props
    renderMonth: null,
    // day props
    modifiers: {},
    renderDay: null,
    onDayClick() { },
    onDayMouseEnter() { },
    onDayMouseLeave() { },
    // accessibility props
    isFocused: false,
    getFirstFocusableDay: null,
    onBlur() { },
    showKeyboardShortcuts: false,
    // internationalization
    monthFormat: 'MMMM YYYY',
    phrases: defaultPhrases_1.DayPickerPhrases,
};
function applyTransformStyles(el, transform, opacity = '') {
    const transformStyles = getTransformStyles_1.default(transform);
    transformStyles.opacity = opacity;
    Object.keys(transformStyles).forEach((styleKey) => {
        // eslint-disable-next-line no-param-reassign
        el.style[styleKey] = transformStyles[styleKey];
    });
}
function calculateDimension(el, axis, borderBox = false, withMargin = false, theme) {
    if (!el) {
        return 0;
    }
    const axisStart = axis === 'width' ? 'Left' : 'Top';
    const axisEnd = axis === 'width' ? 'Right' : 'Bottom';
    // Only read styles if we need to
    const style = (!borderBox || withMargin) ? window.getComputedStyle(el) : null;
    // style = theme[style] ? theme[style] : style;
    // Offset includes border and padding
    const { offsetWidth, offsetHeight } = el;
    let size = axis === 'width' ? offsetWidth : offsetHeight;
    // Get the inner size
    if (!borderBox) {
        size -= (parseFloat(style[`padding${axisStart}`]) +
            parseFloat(style[`padding${axisEnd}`]) +
            parseFloat(style[`border${axisStart}Width`]) +
            parseFloat(style[`border${axisEnd}Width`]));
    }
    // Apply margin
    if (withMargin) {
        size += (parseFloat(style[`margin${axisStart}`]) + parseFloat(style[`margin${axisEnd}`]));
    }
    return size;
}
exports.calculateDimension = calculateDimension;
function getMonthHeight(el, theme) {
    const themedCaption = theme['CalendarMonth__caption'];
    const caption = el.querySelector(`.${themedCaption}`);
    const themedGrid = theme['CalendarMonth__grid'];
    const grid = el.querySelector('tbody');
    // Need to separate out table children for FF
    // Add an additional +1 for the border
    return (calculateDimension(caption, 'height', true, true, theme) + calculateDimension(grid, 'height', false, false, theme) + 1);
}
class DayPicker extends react_1.default.Component {
    constructor(props) {
        super(props);
        // this.getMonthHeightByIndex = this.getMonthHeightByIndex.bind(this);
        // this.adjustDayPickerHeight = this.adjustDayPickerHeight.bind(this);
        const currentMonth = props.hidden ? moment_1.default() : props.initialVisibleMonth();
        let focusedDate = currentMonth.clone().startOf('month');
        if (props.getFirstFocusableDay) {
            focusedDate = props.getFirstFocusableDay(currentMonth);
        }
        const translationValue = props.isRTL && this.isHorizontal() ? -getCalendarMonthWidth_1.default(props.daySize) : 0;
        this.hasSetInitialVisibleMonth = !props.hidden;
        this.state = {
            currentMonth,
            monthTransition: null,
            translationValue,
            scrollableMonthMultiple: 1,
            calendarMonthWidth: getCalendarMonthWidth_1.default(props.daySize),
            focusedDate: (!props.hidden || props.isFocused) ? focusedDate : null,
            nextFocusedDate: null,
            showKeyboardShortcuts: props.showKeyboardShortcuts,
            onKeyboardShortcutsPanelClose() { },
            isTouchDevice: is_touch_device_1.default(),
            withMouseInteractions: true,
        };
        this.onKeyDown = this.onKeyDown.bind(this);
        this.onPrevMonthClick = this.onPrevMonthClick.bind(this);
        this.onNextMonthClick = this.onNextMonthClick.bind(this);
        this.setCalendarMonthGridRef = this.setCalendarMonthGridRef.bind(this);
        this.multiplyScrollableMonths = this.multiplyScrollableMonths.bind(this);
        this.updateStateAfterMonthTransition = this.updateStateAfterMonthTransition.bind(this);
        this.openKeyboardShortcutsPanel = this.openKeyboardShortcutsPanel.bind(this);
        this.closeKeyboardShortcutsPanel = this.closeKeyboardShortcutsPanel.bind(this);
    }
    componentDidMount() {
        this.setState({ isTouchDevice: is_touch_device_1.default() });
        if (this.isHorizontal()) {
            this.adjustDayPickerHeight();
            this.initializeDayPickerWidth();
        }
    }
    componentWillReceiveProps(nextProps) {
        const { hidden, isFocused, showKeyboardShortcuts, onBlur } = nextProps;
        const { currentMonth } = this.state;
        if (!hidden) {
            if (!this.hasSetInitialVisibleMonth) {
                this.hasSetInitialVisibleMonth = true;
                this.setState({
                    currentMonth: nextProps.initialVisibleMonth(),
                });
            }
            if (!this.dayPickerWidth && this.isHorizontal()) {
                this.initializeDayPickerWidth();
                this.adjustDayPickerHeight();
            }
        }
        if (nextProps.daySize !== this.props.daySize) {
            this.setState({
                calendarMonthWidth: getCalendarMonthWidth_1.default(nextProps.daySize),
            });
        }
        if (isFocused !== this.props.isFocused) {
            if (isFocused) {
                const focusedDate = this.getFocusedDay(currentMonth);
                let onKeyboardShortcutsPanelClose = this.state.onKeyboardShortcutsPanelClose;
                if (nextProps.showKeyboardShortcuts) {
                    // the ? shortcut came from the input and we should return input there once it is close
                    onKeyboardShortcutsPanelClose = onBlur;
                }
                this.setState({
                    showKeyboardShortcuts,
                    onKeyboardShortcutsPanelClose,
                    focusedDate,
                    withMouseInteractions: false,
                });
            }
            else {
                this.setState({ focusedDate: null });
            }
        }
    }
    shouldComponentUpdate(nextProps, nextState) {
        return react_addons_shallow_compare_1.default(this, nextProps, nextState);
    }
    componentDidUpdate(prevProps, prevState) {
        const { monthTransition, currentMonth, focusedDate } = this.state;
        if (monthTransition || !currentMonth.isSame(prevState.currentMonth)) {
            if (this.isHorizontal()) {
                this.adjustDayPickerHeight();
            }
        }
        if ((!prevProps.isFocused && this.props.isFocused && !focusedDate) ||
            (!prevProps.showKeyboardShortcuts && this.props.showKeyboardShortcuts)) {
            this.container.focus();
        }
    }
    onKeyDown(e) {
        e.stopPropagation();
        this.setState({ withMouseInteractions: false });
        const { onBlur } = this.props;
        const { focusedDate, showKeyboardShortcuts } = this.state;
        if (!focusedDate)
            return;
        const newFocusedDate = focusedDate.clone();
        let didTransitionMonth = false;
        // focus might be anywhere when the keyboard shortcuts panel is opened so we want to
        // return it to wherever it was before when the panel was opened
        const activeElement = getActiveElement_1.default();
        const onKeyboardShortcutsPanelClose = () => {
            if (activeElement)
                activeElement.focus();
        };
        switch (e.key) {
            case 'ArrowUp':
                e.preventDefault();
                newFocusedDate.subtract(1, 'week');
                didTransitionMonth = this.maybeTransitionPrevMonth(newFocusedDate);
                break;
            case 'ArrowLeft':
                e.preventDefault();
                newFocusedDate.subtract(1, 'day');
                didTransitionMonth = this.maybeTransitionPrevMonth(newFocusedDate);
                break;
            case 'Home':
                e.preventDefault();
                newFocusedDate.startOf('week');
                didTransitionMonth = this.maybeTransitionPrevMonth(newFocusedDate);
                break;
            case 'PageUp':
                e.preventDefault();
                newFocusedDate.subtract(1, 'month');
                didTransitionMonth = this.maybeTransitionPrevMonth(newFocusedDate);
                break;
            case 'ArrowDown':
                e.preventDefault();
                newFocusedDate.add(1, 'week');
                didTransitionMonth = this.maybeTransitionNextMonth(newFocusedDate);
                break;
            case 'ArrowRight':
                e.preventDefault();
                newFocusedDate.add(1, 'day');
                didTransitionMonth = this.maybeTransitionNextMonth(newFocusedDate);
                break;
            case 'End':
                e.preventDefault();
                newFocusedDate.endOf('week');
                didTransitionMonth = this.maybeTransitionNextMonth(newFocusedDate);
                break;
            case 'PageDown':
                e.preventDefault();
                newFocusedDate.add(1, 'month');
                didTransitionMonth = this.maybeTransitionNextMonth(newFocusedDate);
                break;
            case '?':
                this.openKeyboardShortcutsPanel(onKeyboardShortcutsPanelClose);
                break;
            case 'Escape':
                if (showKeyboardShortcuts) {
                    this.closeKeyboardShortcutsPanel();
                }
                else {
                    onBlur();
                }
                break;
            default:
                break;
        }
        // If there was a month transition, do not update the focused date until the transition has
        // completed. Otherwise, attempting to focus on a DOM node may interrupt the CSS animation. If
        // didTransitionMonth is true, the focusedDate gets updated in #updateStateAfterMonthTransition
        if (!didTransitionMonth) {
            this.setState({
                focusedDate: newFocusedDate,
            });
        }
    }
    onPrevMonthClick(nextFocusedDate, e) {
        const { isRTL } = this.props;
        if (e)
            e.preventDefault();
        let translationValue = this.isVertical() ? this.getMonthHeightByIndex(0) : this.dayPickerWidth;
        if (isRTL && this.isHorizontal()) {
            translationValue = -2 * this.dayPickerWidth;
        }
        // The first CalendarMonth is always positioned absolute at top: 0 or left: 0
        // so we need to transform it to the appropriate location before the animation.
        // This behavior is because we would otherwise need a double-render in order to
        // adjust the container position once we had the height the first calendar
        // (ie first draw all the calendar, then in a second render, use the first calendar's
        // height to position the container). Variable calendar heights, amirite? <3 Maja
        this.translateFirstDayPickerForAnimation(translationValue);
        this.setState({
            monthTransition: PREV_TRANSITION,
            translationValue,
            focusedDate: null,
            nextFocusedDate,
        });
    }
    onNextMonthClick(nextFocusedDate, e) {
        const { isRTL } = this.props;
        if (e)
            e.preventDefault();
        let translationValue = this.isVertical() ? -this.getMonthHeightByIndex(1) : -this.dayPickerWidth;
        if (isRTL && this.isHorizontal()) {
            translationValue = 0;
        }
        this.setState({
            monthTransition: NEXT_TRANSITION,
            translationValue,
            focusedDate: null,
            nextFocusedDate,
        });
    }
    getFocusedDay(newMonth) {
        const { getFirstFocusableDay, numberOfMonths } = this.props;
        let focusedDate;
        if (getFirstFocusableDay) {
            focusedDate = getFirstFocusableDay(newMonth);
        }
        if (newMonth && (!focusedDate || !isDayVisible_1.default(focusedDate, newMonth, numberOfMonths))) {
            focusedDate = newMonth.clone().startOf('month');
        }
        return focusedDate;
    }
    getMonthHeightByIndex(i) {
        return getMonthHeight(this.transitionContainer.querySelectorAll(`.${this.props.theme}['CalendarMonth']`)[i], this.props.theme);
    }
    setCalendarMonthGridRef(ref) {
        this.calendarMonthGrid = ref;
    }
    maybeTransitionNextMonth(newFocusedDate) {
        const { numberOfMonths } = this.props;
        const { currentMonth, focusedDate } = this.state;
        const newFocusedDateMonth = newFocusedDate.month();
        const focusedDateMonth = focusedDate.month();
        const isNewFocusedDateVisible = isDayVisible_1.default(newFocusedDate, currentMonth, numberOfMonths);
        if (newFocusedDateMonth !== focusedDateMonth && !isNewFocusedDateVisible) {
            this.onNextMonthClick(newFocusedDate);
            return true;
        }
        return false;
    }
    maybeTransitionPrevMonth(newFocusedDate) {
        const { numberOfMonths } = this.props;
        const { currentMonth, focusedDate } = this.state;
        const newFocusedDateMonth = newFocusedDate.month();
        const focusedDateMonth = focusedDate.month();
        const isNewFocusedDateVisible = isDayVisible_1.default(newFocusedDate, currentMonth, numberOfMonths);
        if (newFocusedDateMonth !== focusedDateMonth && !isNewFocusedDateVisible) {
            this.onPrevMonthClick(newFocusedDate);
            return true;
        }
        return false;
    }
    multiplyScrollableMonths(e) {
        const { onMultiplyScrollableMonths } = this.props;
        if (e)
            e.preventDefault();
        if (onMultiplyScrollableMonths)
            onMultiplyScrollableMonths(e);
        this.setState({
            scrollableMonthMultiple: this.state.scrollableMonthMultiple + 1,
        });
    }
    isHorizontal() {
        return this.props.orientation === Constants_1.HORIZONTAL_ORIENTATION;
    }
    isVertical() {
        return this.props.orientation === Constants_1.VERTICAL_ORIENTATION ||
            this.props.orientation === Constants_1.VERTICAL_SCROLLABLE;
    }
    initializeDayPickerWidth() {
        const themedStyle = this.props.theme['CalendarMonth'];
        if (this.calendarMonthGrid) {
            // eslint-disable-next-line react/no-find-dom-node
            const calendarMonthGridDOMNode = react_dom_1.default.findDOMNode(this.calendarMonthGrid);
            if (calendarMonthGridDOMNode) {
                this.dayPickerWidth = calculateDimension(calendarMonthGridDOMNode.querySelector(`.${themedStyle}`), 'width', true, this.props.theme);
            }
        }
    }
    updateStateAfterMonthTransition() {
        const { onPrevMonthClick, onNextMonthClick, } = this.props;
        const { currentMonth, monthTransition, focusedDate, nextFocusedDate, withMouseInteractions, } = this.state;
        if (!monthTransition)
            return;
        const newMonth = currentMonth.clone();
        if (monthTransition === PREV_TRANSITION) {
            if (onPrevMonthClick)
                onPrevMonthClick();
            newMonth.subtract(1, 'month');
        }
        else if (monthTransition === NEXT_TRANSITION) {
            if (onNextMonthClick)
                onNextMonthClick();
            newMonth.add(1, 'month');
        }
        let newFocusedDate = null;
        if (nextFocusedDate) {
            newFocusedDate = nextFocusedDate;
        }
        else if (!focusedDate && !withMouseInteractions) {
            newFocusedDate = this.getFocusedDay(newMonth);
        }
        if (this.calendarMonthGrid) {
            // eslint-disable-next-line react/no-find-dom-node
            const calendarMonthGridDOMNode = react_dom_1.default.findDOMNode(this.calendarMonthGrid);
            if (calendarMonthGridDOMNode) {
                // clear the previous transforms
                const styledTheme = this.props.theme['CalendarMonth'];
                applyTransformStyles(calendarMonthGridDOMNode.querySelector(`.${styledTheme}`), 'none');
            }
        }
        this.setState({
            currentMonth: newMonth,
            monthTransition: null,
            translationValue: (this.props.isRTL && this.isHorizontal()) ? -this.dayPickerWidth : 0,
            nextFocusedDate: null,
            focusedDate: newFocusedDate,
        }, () => {
            // we don't want to focus on the relevant calendar day after a month transition
            // if the user is navigating around using a mouse
            if (withMouseInteractions) {
                const activeElement = getActiveElement_1.default();
                if (activeElement && activeElement !== document.body) {
                    activeElement.blur();
                }
            }
        });
    }
    adjustDayPickerHeight() {
        const heights = [];
        const themedStyle = this.props.theme['CalendarMonth'];
        Array.prototype.forEach.call(this.transitionContainer.querySelectorAll(`.${themedStyle}`), (el) => {
            if (el.getAttribute('data-visible') === 'true') {
                heights.push(getMonthHeight(el, this.props.theme));
            }
        });
        const newMonthHeight = Math.max(...heights) + MONTH_PADDING;
        if (newMonthHeight !== calculateDimension(this.transitionContainer, 'height', false, false, this.props.theme)) {
            this.monthHeight = newMonthHeight;
            this.transitionContainer.style.height = `${newMonthHeight}px`;
        }
    }
    translateFirstDayPickerForAnimation(translationValue) {
        const { isRTL, theme } = this.props;
        let convertedTranslationValue = -translationValue;
        if (isRTL && this.isHorizontal()) {
            const positiveTranslationValue = Math.abs(translationValue + this.dayPickerWidth);
            convertedTranslationValue = positiveTranslationValue;
        }
        const transformType = this.isVertical() ? 'translateY' : 'translateX';
        const transformValue = `${transformType}(${convertedTranslationValue}px)`;
        const styledTheme = theme['CalendarMonth'];
        applyTransformStyles(this.transitionContainer.querySelector(`.${styledTheme}`), transformValue, 1);
    }
    openKeyboardShortcutsPanel(onCloseCallBack) {
        this.setState({
            showKeyboardShortcuts: true,
            onKeyboardShortcutsPanelClose: onCloseCallBack,
        });
    }
    closeKeyboardShortcutsPanel() {
        const { onKeyboardShortcutsPanelClose } = this.state;
        if (onKeyboardShortcutsPanelClose) {
            onKeyboardShortcutsPanelClose();
        }
        this.setState({
            onKeyboardShortcutsPanelClose: null,
            showKeyboardShortcuts: false,
        });
    }
    renderNavigation() {
        const { navPrev, navNext, orientation, phrases, isRTL, } = this.props;
        let onNextMonthClick;
        if (orientation === Constants_1.VERTICAL_SCROLLABLE) {
            onNextMonthClick = this.multiplyScrollableMonths;
        }
        else {
            onNextMonthClick = (e) => { this.onNextMonthClick(null, e); };
        }
        return (react_1.default.createElement(DayPickerNavigation_1.default, { onPrevMonthClick: (e) => { this.onPrevMonthClick(null, e); }, onNextMonthClick: onNextMonthClick, navPrev: navPrev, navNext: navNext, orientation: orientation, phrases: phrases, isRTL: isRTL }));
    }
    renderWeekHeader(index) {
        const { daySize, orientation, theme } = this.props;
        const { calendarMonthWidth } = this.state;
        const verticalScrollable = orientation === Constants_1.VERTICAL_SCROLLABLE;
        const horizontalStyle = {
            left: index * calendarMonthWidth,
        };
        const verticalStyle = {
            marginLeft: -calendarMonthWidth / 2,
        };
        let style = {}; // no styles applied to the vertical-scrollable orientation
        if (this.isHorizontal()) {
            style = horizontalStyle;
        }
        else if (this.isVertical() && !verticalScrollable) {
            style = verticalStyle;
        }
        let { firstDayOfWeek } = this.props;
        if (firstDayOfWeek == null) {
            firstDayOfWeek = moment_1.default.localeData().firstDayOfWeek();
        }
        const header = [];
        for (let i = 0; i < 7; i += 1) {
            header.push(react_1.default.createElement("li", { key: i, style: { width: daySize } },
                react_1.default.createElement("small", null, moment_1.default().day((i + firstDayOfWeek) % 7).format('dd'))));
        }
        return (react_1.default.createElement("div", { className: theme["DayPicker__week-header"], key: `week-${index}`, style: style },
            react_1.default.createElement("ul", null, header)));
    }
    render() {
        const { calendarMonthWidth, currentMonth, monthTransition, translationValue, scrollableMonthMultiple, focusedDate, showKeyboardShortcuts, isTouchDevice: isTouch, } = this.state;
        const { enableOutsideDays, numberOfMonths, orientation, modifiers, withPortal, onDayClick, onDayMouseEnter, onDayMouseLeave, firstDayOfWeek, renderMonth, renderDay, renderCalendarInfo, hideKeyboardShortcutsPanel, onOutsideClick, monthFormat, daySize, isFocused, phrases, theme, } = this.props;
        const numOfWeekHeaders = this.isVertical() ? 1 : numberOfMonths;
        const weekHeaders = [];
        for (let i = 0; i < numOfWeekHeaders; i += 1) {
            weekHeaders.push(this.renderWeekHeader(i));
        }
        let firstVisibleMonthIndex = 1;
        if (monthTransition === PREV_TRANSITION) {
            firstVisibleMonthIndex -= 1;
        }
        else if (monthTransition === NEXT_TRANSITION) {
            firstVisibleMonthIndex += 1;
        }
        const verticalScrollable = this.props.orientation === Constants_1.VERTICAL_SCROLLABLE;
        if (verticalScrollable)
            firstVisibleMonthIndex = 0;
        const dayPickerClassNames = classnames_1.default(theme['DayPicker'], {
            [theme['DayPicker--horizontal']]: this.isHorizontal(),
            [theme['DayPicker--vertical']]: this.isVertical(),
            [theme['DayPicker--vertical-scrollable']]: verticalScrollable,
            [theme['DayPicker--portal']]: withPortal,
        });
        const transitionContainerClasses = classnames_1.default(theme['transition-container'], {
            [theme['transition-container--horizontal']]: this.isHorizontal(),
            [theme['transition-container--vertical']]: this.isVertical(),
        });
        const horizontalWidth = (calendarMonthWidth * numberOfMonths) + (2 * DAY_PICKER_PADDING);
        // this is a kind of made-up value that generally looks good. we'll
        // probably want to let the user set this explicitly.
        const verticalHeight = 1.75 * calendarMonthWidth;
        const dayPickerStyle = {
            width: this.isHorizontal() && horizontalWidth,
            // These values are to center the datepicker (approximately) on the page
            marginLeft: this.isHorizontal() && withPortal && -horizontalWidth / 2,
            marginTop: this.isHorizontal() && withPortal && -calendarMonthWidth / 2,
        };
        const transitionContainerStyle = {
            width: this.isHorizontal() && horizontalWidth,
            height: this.isVertical() && !verticalScrollable && !withPortal && verticalHeight,
        };
        const isCalendarMonthGridAnimating = monthTransition !== null;
        const transformType = this.isVertical() ? 'translateY' : 'translateX';
        const transformValue = `${transformType}(${translationValue}px)`;
        const shouldFocusDate = !isCalendarMonthGridAnimating && isFocused;
        let keyboardShortcutButtonLocation = DayPickerKeyboardShortcuts_1.BOTTOM_RIGHT;
        if (this.isVertical()) {
            keyboardShortcutButtonLocation = withPortal ? DayPickerKeyboardShortcuts_1.TOP_LEFT : DayPickerKeyboardShortcuts_1.TOP_RIGHT;
        }
        return (react_1.default.createElement("div", { className: dayPickerClassNames, style: dayPickerStyle },
            react_1.default.createElement(OutsideClickHandler_1.default, { onOutsideClick: onOutsideClick },
                react_1.default.createElement("div", { className: theme["DayPicker__week-headers"], "aria-hidden": "true", role: "presentation" }, weekHeaders),
                react_1.default.createElement("div", { className: theme["DayPicker__focus-region"], ref: (ref) => { this.container = ref; }, onClick: (e) => { e.stopPropagation(); }, onKeyDown: throttle_1.default(this.onKeyDown, 300), onMouseUp: () => { this.setState({ withMouseInteractions: true }); }, role: "region", tabIndex: -1 },
                    !verticalScrollable && this.renderNavigation(),
                    react_1.default.createElement("div", { className: transitionContainerClasses, ref: (ref) => { this.transitionContainer = ref; }, style: transitionContainerStyle },
                        react_1.default.createElement(CalendarMonthGrid_1.default, { ref: this.setCalendarMonthGridRef, transformValue: transformValue, enableOutsideDays: enableOutsideDays, firstVisibleMonthIndex: firstVisibleMonthIndex, initialMonth: currentMonth, isAnimating: isCalendarMonthGridAnimating, modifiers: modifiers, orientation: orientation, numberOfMonths: numberOfMonths * scrollableMonthMultiple, onDayClick: onDayClick, onDayMouseEnter: onDayMouseEnter, onDayMouseLeave: onDayMouseLeave, renderMonth: renderMonth, renderDay: renderDay, onMonthTransitionEnd: this.updateStateAfterMonthTransition, monthFormat: monthFormat, daySize: daySize, firstDayOfWeek: firstDayOfWeek, isFocused: shouldFocusDate, focusedDate: focusedDate, phrases: phrases }),
                        verticalScrollable && this.renderNavigation()),
                    !isTouch && !hideKeyboardShortcutsPanel &&
                        react_1.default.createElement(DayPickerKeyboardShortcuts_1.default, { block: this.isVertical() && !withPortal, buttonLocation: keyboardShortcutButtonLocation, showKeyboardShortcutsPanel: showKeyboardShortcuts, openKeyboardShortcutsPanel: this.openKeyboardShortcutsPanel, closeKeyboardShortcutsPanel: this.closeKeyboardShortcutsPanel, phrases: phrases, testingThis: this })),
                renderCalendarInfo && renderCalendarInfo())));
    }
}
exports.default = react_css_themr_1.themr(ThemeIdentifiers_1.DATEPICKER, baseTheme)(DayPicker);
DayPicker.propTypes = propTypes;
DayPicker.defaultProps = exports.defaultProps;
