"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const moment_1 = require("moment");
const classnames_1 = require("classnames");
const react_portal_1 = require("react-portal");
const airbnb_prop_types_1 = require("airbnb-prop-types");
const consolidated_events_1 = require("consolidated-events");
const is_touch_device_1 = require("is-touch-device");
const react_css_themr_1 = require("react-css-themr");
const SingleDatePickerShape_1 = require("../shapes/SingleDatePickerShape");
const defaultPhrases_1 = require("../defaultPhrases");
const OutsideClickHandler_1 = require("./OutsideClickHandler");
const toMomentObject_1 = require("../utils/toMomentObject");
const toLocalizedDateString_1 = require("../utils/toLocalizedDateString");
const getResponsiveContainerStyles_1 = require("../utils/getResponsiveContainerStyles");
const ThemeIdentifiers_1 = require("./../../ThemeIdentifiers");
const baseTheme = require("./../style/style.scss");
const toISODateString_1 = require("../utils/toISODateString");
const SingleDatePickerInput_1 = require("./SingleDatePickerInput");
const DayPickerSingleDateController_1 = require("./DayPickerSingleDateController");
const close_svg_1 = require("../svg/close.svg");
const isInclusivelyAfterDay_1 = require("../utils/isInclusivelyAfterDay");
const Constants_1 = require("../Constants");
const propTypes = airbnb_prop_types_1.forbidExtraProps(SingleDatePickerShape_1.default);
const defaultProps = {
    // required props for a functional interactive SingleDatePicker
    date: null,
    focused: false,
    // input related props
    id: 'date',
    placeholder: 'Date',
    disabled: false,
    required: false,
    readOnly: false,
    screenReaderInputMessage: '',
    showClearDate: false,
    showDefaultInputIcon: false,
    customInputIcon: null,
    customCloseIcon: null,
    // calendar presentation and interaction related props
    orientation: Constants_1.HORIZONTAL_ORIENTATION,
    anchorDirection: Constants_1.ANCHOR_LEFT,
    horizontalMargin: 0,
    withPortal: false,
    withFullScreenPortal: false,
    initialVisibleMonth: null,
    firstDayOfWeek: null,
    numberOfMonths: 2,
    keepOpenOnDateSelect: false,
    reopenPickerOnClearDate: false,
    renderCalendarInfo: null,
    hideKeyboardShortcutsPanel: false,
    daySize: Constants_1.DAY_SIZE,
    isRTL: false,
    // navigation related props
    navPrev: null,
    navNext: null,
    onPrevMonthClick() { },
    onNextMonthClick() { },
    onClose() { },
    // month presentation and interaction related props
    renderMonth: null,
    // day presentation and interaction related props
    renderDay: null,
    enableOutsideDays: false,
    isDayBlocked: () => false,
    isOutsideRange: day => !isInclusivelyAfterDay_1.default(day, moment_1.default()),
    isDayHighlighted: () => { },
    // internationalization props
    displayFormat: () => moment_1.default.localeData().longDateFormat('L'),
    monthFormat: 'MMMM YYYY',
    phrases: defaultPhrases_1.SingleDatePickerPhrases,
};
class SingleDatePicker extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.isTouchDevice = false;
        this.state = {
            dayPickerContainerStyles: {},
            isDayPickerFocused: false,
            isInputFocused: false,
        };
        this.onDayPickerFocus = this.onDayPickerFocus.bind(this);
        this.onDayPickerBlur = this.onDayPickerBlur.bind(this);
        this.onChange = this.onChange.bind(this);
        this.onFocus = this.onFocus.bind(this);
        this.onClearFocus = this.onClearFocus.bind(this);
        this.clearDate = this.clearDate.bind(this);
        this.responsivizePickerPosition = this.responsivizePickerPosition.bind(this);
    }
    /* istanbul ignore next */
    componentDidMount() {
        this.resizeHandle = consolidated_events_1.addEventListener(window, 'resize', this.responsivizePickerPosition, { passive: true });
        this.responsivizePickerPosition();
        if (this.props.focused) {
            this.setState({
                isInputFocused: true,
            });
        }
        this.isTouchDevice = is_touch_device_1.default();
    }
    componentDidUpdate(prevProps) {
        if (!prevProps.focused && this.props.focused) {
            this.responsivizePickerPosition();
        }
    }
    /* istanbul ignore next */
    componentWillUnmount() {
        consolidated_events_1.removeEventListener(this.resizeHandle);
    }
    onChange(dateString) {
        const { isOutsideRange, keepOpenOnDateSelect, onDateChange, onFocusChange, onClose, } = this.props;
        const newDate = toMomentObject_1.default(dateString, this.getDisplayFormat());
        const isValid = newDate && !isOutsideRange(newDate);
        if (isValid) {
            onDateChange(newDate);
            if (!keepOpenOnDateSelect) {
                onFocusChange({ focused: false });
                onClose({ date: newDate });
            }
        }
        else {
            onDateChange(null);
        }
    }
    onFocus() {
        const { disabled, onFocusChange, withPortal, withFullScreenPortal } = this.props;
        const moveFocusToDayPicker = withPortal || withFullScreenPortal || this.isTouchDevice;
        if (moveFocusToDayPicker) {
            this.onDayPickerFocus();
        }
        else {
            this.onDayPickerBlur();
        }
        if (!disabled) {
            onFocusChange({ focused: true });
        }
    }
    onClearFocus() {
        const { startDate, endDate, focused, onFocusChange, onClose } = this.props;
        if (!focused)
            return;
        this.setState({
            isInputFocused: false,
            isDayPickerFocused: false,
        });
        onFocusChange({ focused: false });
        onClose({ startDate, endDate });
    }
    onDayPickerFocus() {
        this.setState({
            isInputFocused: false,
            isDayPickerFocused: true,
        });
    }
    onDayPickerBlur() {
        this.setState({
            isInputFocused: true,
            isDayPickerFocused: false,
        });
    }
    getDateString(date) {
        const displayFormat = this.getDisplayFormat();
        if (date && displayFormat) {
            return date && date.format(displayFormat);
        }
        return toLocalizedDateString_1.default(date);
    }
    getDayPickerContainerClasses() {
        const { orientation, withPortal, withFullScreenPortal, anchorDirection, isRTL, theme } = this.props;
        const dayPickerClassName = classnames_1.default(theme['SingleDatePicker__picker'], {
            [theme['SingleDatePicker__picker--direction-left']]: anchorDirection === Constants_1.ANCHOR_LEFT,
            [theme['SingleDatePicker__picker--direction-right']]: anchorDirection === Constants_1.ANCHOR_RIGHT,
            [theme['SingleDatePicker__picker--horizontal']]: orientation === Constants_1.HORIZONTAL_ORIENTATION,
            [theme['SingleDatePicker__picker--vertical']]: orientation === Constants_1.VERTICAL_ORIENTATION,
            [theme['SingleDatePicker__picker--portal']]: withPortal || withFullScreenPortal,
            [theme['SingleDatePicker__picker--full-screen-portal']]: withFullScreenPortal,
            [theme['SingleDatePicker__picker--rtl']]: isRTL,
        });
        return dayPickerClassName;
    }
    getDisplayFormat() {
        const { displayFormat } = this.props;
        return typeof displayFormat === 'string' ? displayFormat : displayFormat();
    }
    clearDate() {
        const { onDateChange, reopenPickerOnClearDate, onFocusChange } = this.props;
        onDateChange(null);
        if (reopenPickerOnClearDate) {
            onFocusChange({ focused: true });
        }
    }
    /* istanbul ignore next */
    responsivizePickerPosition() {
        const { anchorDirection, horizontalMargin, withPortal, withFullScreenPortal, focused, } = this.props;
        const { dayPickerContainerStyles } = this.state;
        if (!focused) {
            return;
        }
        const isAnchoredLeft = anchorDirection === Constants_1.ANCHOR_LEFT;
        if (!withPortal && !withFullScreenPortal) {
            const containerRect = this.dayPickerContainer.getBoundingClientRect();
            const currentOffset = dayPickerContainerStyles[anchorDirection] || 0;
            const containerEdge = isAnchoredLeft ? containerRect[Constants_1.ANCHOR_RIGHT] : containerRect[Constants_1.ANCHOR_LEFT];
            this.setState({
                dayPickerContainerStyles: getResponsiveContainerStyles_1.default(anchorDirection, currentOffset, containerEdge, horizontalMargin),
            });
        }
    }
    maybeRenderDayPickerWithPortal() {
        const { focused, withPortal, withFullScreenPortal } = this.props;
        if (!focused) {
            return null;
        }
        if (withPortal || withFullScreenPortal) {
            return (react_1.default.createElement(react_portal_1.default, { isOpened: true }, this.renderDayPicker()));
        }
        return this.renderDayPicker();
    }
    renderDayPicker() {
        const { onDateChange, date, onFocusChange, focused, enableOutsideDays, numberOfMonths, orientation, monthFormat, navPrev, navNext, withPortal, withFullScreenPortal, keepOpenOnDateSelect, initialVisibleMonth, renderMonth, renderDay, renderCalendarInfo, hideKeyboardShortcutsPanel, firstDayOfWeek, customCloseIcon, phrases, daySize, isRTL, isOutsideRange, isDayBlocked, isDayHighlighted, } = this.props;
        const { dayPickerContainerStyles, isDayPickerFocused } = this.state;
        const onOutsideClick = (!withFullScreenPortal && withPortal) ? this.onClearFocus : undefined;
        const closeIcon = customCloseIcon || (react_1.default.createElement(close_svg_1.default, null));
        return (react_1.default.createElement("div", { ref: (ref) => { this.dayPickerContainer = ref; }, className: this.getDayPickerContainerClasses(), style: dayPickerContainerStyles, onClick: onOutsideClick },
            react_1.default.createElement(DayPickerSingleDateController_1.default, { date: date, onDateChange: onDateChange, onFocusChange: onFocusChange, orientation: orientation, enableOutsideDays: enableOutsideDays, numberOfMonths: numberOfMonths, monthFormat: monthFormat, withPortal: withPortal || withFullScreenPortal, focused: focused, keepOpenOnDateSelect: keepOpenOnDateSelect, hideKeyboardShortcutsPanel: hideKeyboardShortcutsPanel, initialVisibleMonth: initialVisibleMonth, navPrev: navPrev, navNext: navNext, renderMonth: renderMonth, renderDay: renderDay, renderCalendarInfo: renderCalendarInfo, isFocused: isDayPickerFocused, phrases: phrases, daySize: daySize, isRTL: isRTL, isOutsideRange: isOutsideRange, isDayBlocked: isDayBlocked, isDayHighlighted: isDayHighlighted, firstDayOfWeek: firstDayOfWeek }),
            withFullScreenPortal && (react_1.default.createElement("button", { "aria-label": phrases.closeDatePicker, className: theme["SingleDatePicker__close"], type: "button", onClick: this.onClearFocus },
                react_1.default.createElement("div", { className: theme["SingleDatePicker__close-icon"] }, closeIcon)))));
    }
    render() {
        const { id, placeholder, disabled, focused, required, readOnly, showClearDate, showDefaultInputIcon, customInputIcon, date, phrases, withPortal, withFullScreenPortal, screenReaderInputMessage, isRTL, theme, } = this.props;
        const { isInputFocused } = this.state;
        const displayValue = this.getDateString(date);
        const inputValue = toISODateString_1.default(date);
        const onOutsideClick = (!withPortal && !withFullScreenPortal) ? this.onClearFocus : undefined;
        return (react_1.default.createElement("div", { id: "joemama", className: theme["SingleDatePicker"] },
            react_1.default.createElement(OutsideClickHandler_1.default, { onOutsideClick: onOutsideClick },
                react_1.default.createElement(SingleDatePickerInput_1.default, { id: id, placeholder: placeholder, focused: focused, isFocused: isInputFocused, disabled: disabled, required: required, readOnly: readOnly, showCaret: !withPortal && !withFullScreenPortal, onClearDate: this.clearDate, showClearDate: showClearDate, showDefaultInputIcon: showDefaultInputIcon, customInputIcon: customInputIcon, displayValue: displayValue, inputValue: inputValue, onChange: this.onChange, onFocus: this.onFocus, onKeyDownShiftTab: this.onClearFocus, onKeyDownTab: this.onClearFocus, onKeyDownArrowDown: this.onDayPickerFocus, screenReaderMessage: screenReaderInputMessage, phrases: phrases, isRTL: isRTL }),
                this.maybeRenderDayPickerWithPortal())));
    }
}
exports.default = react_css_themr_1.themr(ThemeIdentifiers_1.DATEPICKER, baseTheme)(SingleDatePicker);
SingleDatePicker.propTypes = propTypes;
SingleDatePicker.defaultProps = defaultProps;
