"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const enzyme_1 = require("enzyme");
const Panel_1 = require("../Panel");
const Video_1 = require("../../Video");
describe('<Panel />', () => {
    const theme = {
        panel: 'thm-pnl',
        heading: 'thm-hdr',
        body: 'thm-body',
    };
    const videoCmp = React.createElement(Video_1.default, { poster: new URL('https://www.testimage.com/imgage.png'), src: [{
                src: 'http://www.yahoo.com/video3',
                type: Video_1.VideoType.MP4,
            }] });
    describe('when default props are provided', () => {
        it('panel should be rendered with default props', () => {
            const subject = enzyme_1.mount(React.createElement(Panel_1.default, { heading: "Panel" }));
            expect(subject.find('div')).toHaveLength(3);
            expect(subject.find('div').at(1).text()).toBe('Panel');
        });
    });
    describe('heading property', () => {
        describe('when set as string', () => {
            it('panel should have header', () => {
                const subject = enzyme_1.mount(React.createElement(Panel_1.default, { heading: "Panel" }));
                expect(subject.find('div')).toHaveLength(3);
                expect(subject.find('div').at(1).text()).toBe('Panel');
            });
        });
        describe('when set as react component', () => {
            const headCmp = React.createElement("h3", null, "Panel");
            it('panel should have header', () => {
                const subject = enzyme_1.mount(React.createElement(Panel_1.default, { heading: headCmp }));
                expect(subject.find('div')).toHaveLength(2);
                expect(subject.find('h3').text()).toBe('Panel');
            });
        });
    });
    describe('video property', () => {
        describe('when not set', () => {
            it('panel should not have video', () => {
                const subject = enzyme_1.mount(React.createElement(Panel_1.default, { heading: "Panel" }));
                expect(subject.find('div')).toHaveLength(3);
                expect(subject.find('div').at(1).text()).toBe('Panel');
                expect(subject.find('div').at(2).children()).toHaveLength(0);
            });
        });
        describe('when set', () => {
            it('panel should have video', () => {
                const subject = enzyme_1.mount(React.createElement(Panel_1.default, { heading: "Panel with Video", video: videoCmp }));
                expect(subject.find('div')).toHaveLength(3);
                expect(subject.find('div').at(1).text()).toBe('Panel with Video');
                expect(subject.find('div').at(2).children().html()).toBe('<video poster=\"https://www.testimage.com/imgage.png\"><source src=\"http://www.yahoo.com/video3\" type=\"video/mp4\"></video>');
            });
        });
    });
    describe('children property', () => {
        describe('when not set', () => {
            it('panel should render without children container', () => {
                const subject = enzyme_1.mount(React.createElement(Panel_1.default, { heading: "Panel" }));
                expect(subject.find('div')).toHaveLength(3);
                expect(subject.find('div').at(1).text()).toBe('Panel');
                expect(subject.find('div').at(2).children()).toHaveLength(0);
            });
        });
        describe('when set', () => {
            it('panel should render children', () => {
                const subject = enzyme_1.mount(React.createElement(Panel_1.default, { heading: "Panel" },
                    React.createElement("p", null, "This is a paragraph!")));
                expect(subject.find('div')).toHaveLength(3);
                expect(subject.find('div').at(1).text()).toBe('Panel');
                expect(subject.find('div').at(2).children().html()).toBe('<p>This is a paragraph!</p>');
            });
        });
    });
    describe('theme property', () => {
        describe('when not set', () => {
            it('panel should render without theme css', () => {
                const subject = enzyme_1.mount(React.createElement(Panel_1.default, { heading: "Panel" }));
                expect(subject.find('div')).toHaveLength(3);
                expect(subject.find('div').at(1).text()).toBe('Panel');
                expect(subject.html()).toBe('<div><div>Panel</div><div></div></div>');
            });
        });
        describe('when set', () => {
            it('panel should render theme css', () => {
                const subject = enzyme_1.mount(React.createElement(Panel_1.default, { heading: "Panel", theme: theme }, "ABC"));
                expect(subject.find('div')).toHaveLength(3);
                expect(subject.find('div').at(1).text()).toBe('Panel');
                expect(subject.find('div').at(0).hasClass('thm-pnl')).toBe(true);
                expect(subject.find('div').at(1).hasClass('thm-hdr')).toBe(true);
                expect(subject.find('div').at(2).hasClass('thm-body')).toBe(true);
            });
        });
    });
    describe('all property', () => {
        describe('when set', () => {
            it('panel should render using all props', () => {
                const subject = enzyme_1.mount(React.createElement(Panel_1.default, { heading: "Panel", video: videoCmp, theme: theme },
                    React.createElement("p", null, "test")));
                expect(subject.find('div')).toHaveLength(3);
                expect(subject.find('div').at(1).text()).toBe('Panel');
                expect(subject.find('div').at(2).children()).toHaveLength(2);
                expect(subject.find('div').at(2).html()).toBe('<div class=\"thm-body\"><video poster=\"https://www.testimage.com/imgage.png\"><source src=\"http://www.yahoo.com/video3\" type=\"video/mp4\"></video><p>test</p></div>');
                expect(subject.find('div').at(0).hasClass('thm-pnl')).toBe(true);
                expect(subject.find('div').at(1).hasClass('thm-hdr')).toBe(true);
                expect(subject.find('div').at(2).hasClass('thm-body')).toBe(true);
            });
        });
    });
});
